@@ -41,6 +41,7 @@ do {                                                                  \
 static snd_output_t *snd_output = NULL;
 static char *default_device = "default";
 static char *alsa_device = NULL;
+static _AL_LIST* output_device_list;
 
 // TODO: Setting this to 256 causes (extreme, about than 10 seconds)
 // lag if the alsa device is really pulseaudio.
@@ -140,6 +141,8 @@ Error:
    other processes to use the device */
 static void alsa_close(void)
 {
+   _al_list_destroy(output_device_list);
+
    if (alsa_device != default_device)
       al_free(alsa_device);
    
@@ -875,6 +878,125 @@ static void alsa_deallocate_recorder(ALLEGRO_AUDIO_RECORDER *r)
    snd_pcm_close(data->capture_handle);
 }
 
+static void _output_device_list_dtor(void* value, void* userdata)
+{
+   (void)userdata;
+
+   ALLEGRO_AUDIO_DEVICE* device = (ALLEGRO_AUDIO_DEVICE*)value;
+   al_free(device->name);
+   al_free(device->identifier);
+   al_free(device);
+}
+
+static _AL_LIST* alsa_get_output_devices(void)
+{
+   int res;
+   if (!output_device_list) {
+      output_device_list = _al_list_create();
+
+      int rcard = -1;
+      while(snd_card_next(&rcard) == 0) {
+         if (rcard < 0) {
+            ALLEGRO_WARN("snd_card_next returned a bad card %d.\n", rcard);
+            /* Something is really messed up, so bail out. */
+            return output_device_list;
+         }
+
+         snd_ctl_t *handle;
+         char str[64];
+
+         snprintf(str, sizeof str, "hw:%i", rcard);
+         res = snd_ctl_open(&handle, str, 0);
+         if (res < 0) {
+            ALLEGRO_WARN("snd_ctl_open on '%s' failed with code %d.\n", str, res);
+            continue;
+         }
+
+         snd_ctl_card_info_t *card_info;
+         snd_ctl_card_info_alloca(&card_info);
+         res = snd_ctl_card_info(handle, card_info);
+         if (res < 0) {
+            ALLEGRO_WARN("snd_ctl_card_info on '%s' failed with code %d.\n", str, res);
+            continue;
+         }
+
+         const char* card_name = snd_ctl_card_info_get_name(card_info);
+
+         int dev_num = -1;
+
+         while(1) {
+            if (snd_ctl_pcm_next_device(handle, &dev_num) < 0) {
+               break;
+            }
+
+            if (dev_num < 0) {
+               ALLEGRO_WARN("Invalid device number for card %d.\n", rcard);
+               break;
+            }
+
+            void **hints;
+            res = snd_device_name_hint(dev_num, "pcm", &hints);
+            if (res < 0) {
+               ALLEGRO_WARN("snd_device_name_hint for card %d, device %d failed with error code %d\n",
+                  rcard, dev_num, res);
+               continue;
+            }
+            char**n = (char**)hints;
+
+            char* identifier = 0;
+            char* name = 0;
+            while(*n != NULL)
+            {
+               identifier = snd_device_name_get_hint(*n, "NAME");
+               name = snd_device_name_get_hint(*n, "DESC");
+
+               char* ioid = snd_device_name_get_hint(*n, "IOID");
+               if (ioid == NULL || strcmp(ioid, "Output") == 0)
+               {
+                  free(ioid);
+                  break;
+               }
+               free(ioid);
+               n++;
+            }
+
+            if (!identifier || !name) {
+               snd_device_name_free_hint(hints);
+               free(identifier);
+               free(name);
+               ALLEGRO_WARN("Could not find identifier/name for card %d, device %d\n",
+                  rcard, dev_num);
+               continue;
+            }
+
+            char* sep_at = strchr(name, '\n');
+            char* actual_name = sep_at ? sep_at + 1 : name;
+            
+            /* Two string lengths + space + nul terminator. */
+            int len = strlen(actual_name) + strlen(card_name) + 1 + 1;
+            int identifier_len = strlen(identifier) + 1;
+
+            ALLEGRO_AUDIO_DEVICE* device = (ALLEGRO_AUDIO_DEVICE*)al_malloc(sizeof(ALLEGRO_AUDIO_DEVICE));
+            device->identifier = (char*)al_malloc(identifier_len);
+            device->name = (char*)al_malloc(len);
+
+            snprintf(device->name, len, "%s %s", card_name, actual_name);
+            strcpy(device->identifier, identifier);
+
+            snd_device_name_free_hint(hints);
+            free(identifier);
+            free(name);
+
+            _al_list_push_back_ex(output_device_list, device, _output_device_list_dtor);
+         }
+
+         snd_ctl_close(handle);
+      }
+   }
+
+   return output_device_list;
+}
+
 ALLEGRO_AUDIO_DRIVER _al_kcm_alsa_driver =
 {
    "ALSA",
@@ -897,7 +1019,9 @@ ALLEGRO_AUDIO_DRIVER _al_kcm_alsa_driver =
    alsa_set_voice_position,
 
    alsa_allocate_recorder,
-   alsa_deallocate_recorder
+   alsa_deallocate_recorder,
+
+   alsa_get_output_devices
 };
 
 /* vim: set sts=3 sw=3 et: */
