@@ -13,7 +13,7 @@
  *      By Tim Gunn.
  *
  *      RLE support added by Michal Mertl and Salvador Eduardo Tropea.
- * 
+ *
  *      Palette reading improved by Peter Wang.
  *
  *      Big-endian support added by Eric Botcazou.
@@ -34,6 +34,7 @@ ALLEGRO_DEBUG_CHANNEL("image")
 
 /* raw_tga_read8:
  *  Helper for reading 256-color raw data from TGA files.
+ *  Returns pointer past the end.
  */
 static INLINE unsigned char *raw_tga_read8(unsigned char *b, int w, ALLEGRO_FILE *f)
 {
@@ -44,8 +45,9 @@ static INLINE unsigned char *raw_tga_read8(unsigned char *b, int w, ALLEGRO_FILE
 
 /* rle_tga_read8:
  *  Helper for reading 256-color RLE data from TGA files.
+ *  Returns pointer past the end or NULL for error.
  */
-static void rle_tga_read8(unsigned char *b, int w, ALLEGRO_FILE *f)
+static unsigned char *rle_tga_read8(unsigned char *b, int w, ALLEGRO_FILE *f)
 {
    int value, count, c = 0;
 
@@ -55,6 +57,10 @@ static void rle_tga_read8(unsigned char *b, int w, ALLEGRO_FILE *f)
          /* run-length packet */
          count = (count & 0x7F) + 1;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          value = al_fgetc(f);
          while (count--)
             *b++ = value;
@@ -63,9 +69,14 @@ static void rle_tga_read8(unsigned char *b, int w, ALLEGRO_FILE *f)
          /* raw packet */
          count++;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          b = raw_tga_read8(b, count, f);
       }
    } while (c < w);
+   return b;
 }
 
 
@@ -82,6 +93,7 @@ static INLINE int32_t single_tga_read32(ALLEGRO_FILE *f)
 
 /* raw_tga_read32:
  *  Helper for reading 32-bit raw data from TGA files.
+ *  Returns pointer past the end.
  */
 static unsigned int *raw_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
 {
@@ -95,8 +107,9 @@ static unsigned int *raw_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
 
 /* rle_tga_read32:
  *  Helper for reading 32-bit RLE data from TGA files.
+ *  Returns pointer past the end or NULL for error.
  */
-static void rle_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
+static unsigned int *rle_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
 {
    int color, count, c = 0;
 
@@ -106,6 +119,10 @@ static void rle_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
          /* run-length packet */
          count = (count & 0x7F) + 1;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          color = single_tga_read32(f);
          while (count--)
             *b++ = color;
@@ -114,9 +131,14 @@ static void rle_tga_read32(unsigned int *b, int w, ALLEGRO_FILE *f)
          /* raw packet */
          count++;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          b = raw_tga_read32(b, count, f);
       }
    } while (c < w);
+   return b;
 }
 
 
@@ -132,6 +154,7 @@ static INLINE void single_tga_read24(ALLEGRO_FILE *f, unsigned char color[3])
 
 /* raw_tga_read24:
  *  Helper for reading 24-bit raw data from TGA files.
+ *  Returns pointer past the end.
  */
 static unsigned char *raw_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
 {
@@ -147,8 +170,9 @@ static unsigned char *raw_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
 
 /* rle_tga_read24:
  *  Helper for reading 24-bit RLE data from TGA files.
+ *  Returns pointer past the end or NULL for error.
  */
-static void rle_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
+static unsigned char *rle_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
 {
    int count, c = 0;
    unsigned char color[3];
@@ -159,6 +183,10 @@ static void rle_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
          /* run-length packet */
          count = (count & 0x7F) + 1;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          single_tga_read24(f, color);
          while (count--) {
             b[0] = color[0];
@@ -171,9 +199,14 @@ static void rle_tga_read24(unsigned char *b, int w, ALLEGRO_FILE *f)
          /* raw packet */
          count++;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          b = raw_tga_read24(b, count, f);
       }
    } while (c < w);
+   return b;
 }
 
 
@@ -190,6 +223,7 @@ static INLINE int single_tga_read16(ALLEGRO_FILE *f)
 
 /* raw_tga_read16:
  *  Helper for reading 16-bit raw data from TGA files.
+ *  Returns pointer past the end.
  */
 static unsigned short *raw_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
 {
@@ -203,8 +237,9 @@ static unsigned short *raw_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
 
 /* rle_tga_read16:
  *  Helper for reading 16-bit RLE data from TGA files.
+ *  Returns pointer past the end or NULL for error.
  */
-static void rle_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
+static unsigned short *rle_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
 {
    int color, count, c = 0;
 
@@ -214,6 +249,10 @@ static void rle_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
          /* run-length packet */
          count = (count & 0x7F) + 1;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          color = single_tga_read16(f);
          while (count--)
             *b++ = color;
@@ -222,11 +261,17 @@ static void rle_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
          /* raw packet */
          count++;
          c += count;
+         if (c > w) {
+            /* Stepped past the end of the line, error */
+            return NULL;
+         }
          b = raw_tga_read16(b, count, f);
       }
    } while (c < w);
+   return b;
 }
 
+typedef unsigned char palette_entry[3];
 
 /* Like load_tga, but starts loading from the current place in the ALLEGRO_FILE
  *  specified. If successful the offset into the file will be left just after
@@ -236,10 +281,12 @@ static void rle_tga_read16(unsigned short *b, int w, ALLEGRO_FILE *f)
  */
 ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
 {
-   unsigned char image_id[256], image_palette[256][3];
+   unsigned char image_id[256];
+   palette_entry image_palette[256];
    unsigned char id_length, palette_type, image_type, palette_entry_size;
    unsigned char bpp, descriptor_bits;
    short unsigned int palette_colors;
+   short unsigned int palette_start;
    short unsigned int image_width, image_height;
    bool left_to_right;
    bool top_to_bottom;
@@ -255,7 +302,7 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
    id_length = al_fgetc(f);
    palette_type = al_fgetc(f);
    image_type = al_fgetc(f);
-   al_fread16le(f); /* first_color */
+   palette_start = al_fread16le(f); /* first_color */
    palette_colors  = al_fread16le(f);
    palette_entry_size = al_fgetc(f);
    al_fread16le(f); /* left */
@@ -270,35 +317,6 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
 
    al_fread(f, image_id, id_length);
 
-   if (palette_type == 1) {
-
-      for (i = 0; i < palette_colors; i++) {
-
-         switch (palette_entry_size) {
-
-            case 16:
-               c = al_fread16le(f);
-               image_palette[i][0] = (c & 0x1F) << 3;
-               image_palette[i][1] = ((c >> 5) & 0x1F) << 3;
-               image_palette[i][2] = ((c >> 10) & 0x1F) << 3;
-               break;
-
-            case 24:
-            case 32:
-               image_palette[i][0] = al_fgetc(f);
-               image_palette[i][1] = al_fgetc(f);
-               image_palette[i][2] = al_fgetc(f);
-               if (palette_entry_size == 32)
-                  al_fgetc(f);
-               break;
-         }
-      }
-   }
-   else if (palette_type != 0) {
-      ALLEGRO_ERROR("Invalid palette type %d.\n", palette_type);
-      return NULL;
-   }
-
    /* Image type:
     *    0 = no image data
     *    1 = uncompressed color mapped
@@ -320,7 +338,9 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
 
       case 1:
          /* paletted image */
-         if ((palette_type != 1) || (bpp != 8)) {
+         /* Only support 8 bit palettes (up to 256 entries) though in principle the file format could have more(?)*/
+         if ((palette_type != 1) || (bpp != 8) || (palette_start + palette_colors) > 256) {
+            ALLEGRO_ERROR("Invalid image/palette/bpp combination %d/%d/%d.\n", image_type, palette_type, bpp);
             return NULL;
          }
 
@@ -334,7 +354,7 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
          else if ((palette_type == 0) && ((bpp == 24) || (bpp == 32))) {
          }
          else {
-            ALLEGRO_ERROR("Invalid palette/image/bpp combination %d/%d/%d.\n", image_type, palette_type, bpp);
+            ALLEGRO_ERROR("Invalid image/palette/bpp combination %d/%d/%d.\n", image_type, palette_type, bpp);
             return NULL;
          }
          break;
@@ -342,7 +362,7 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
       case 3:
          /* grayscale image */
          if ((palette_type != 0) || (bpp != 8)) {
-            ALLEGRO_ERROR("Invalid palette/image/bpp combination %d/%d/%d.\n", image_type, palette_type, bpp);
+            ALLEGRO_ERROR("Invalid image/palette/bpp combination %d/%d/%d.\n", image_type, palette_type, bpp);
             return NULL;
          }
 
@@ -358,6 +378,38 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
          return NULL;
    }
 
+   if (palette_type == 0) {
+      /* No-op */
+   } else if (palette_type == 1) {
+      for (i = 0; i < palette_colors; i++) {
+         palette_entry *entry = image_palette + palette_start + i;
+         switch (palette_entry_size) {
+            case 16:
+               c = al_fread16le(f);
+               (*entry)[0] = (c & 0x1F) << 3;
+               (*entry)[1] = ((c >> 5) & 0x1F) << 3;
+               (*entry)[2] = ((c >> 10) & 0x1F) << 3;
+               break;
+            case 24:
+               (*entry)[0] = al_fgetc(f);
+               (*entry)[1] = al_fgetc(f);
+               (*entry)[2] = al_fgetc(f);
+               break;
+            case 32:
+               (*entry)[0] = al_fgetc(f);
+               (*entry)[1] = al_fgetc(f);
+               (*entry)[2] = al_fgetc(f);
+               al_fgetc(f); /* Ignore 4th byte (alpha) */
+               break;
+         }
+      }
+   }
+   else  {
+      ALLEGRO_ERROR("Invalid palette type %d.\n", palette_type);
+      return NULL;
+   }
+
+
    bmp = al_create_bitmap(image_width, image_height);
    if (!bmp) {
       ALLEGRO_ERROR("Failed to create bitmap.\n");
@@ -388,21 +440,37 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
       switch (image_type) {
 
          case 1:
-         case 3:
+         case 3: {
+            unsigned char *ptr;
             if (compressed)
-               rle_tga_read8(buf, image_width, f);
+               ptr = rle_tga_read8(buf, image_width, f);
             else
-               raw_tga_read8(buf, image_width, f);
-
+               ptr = raw_tga_read8(buf, image_width, f);
+            if (!ptr) {
+               al_free(buf);
+               al_unlock_bitmap(bmp);
+               al_destroy_bitmap(bmp);
+               ALLEGRO_ERROR("Invalid image data.\n");
+               return NULL;
+            }
+         }
             for (i = 0; i < image_width; i++) {
                int true_x = (left_to_right) ? i : (image_width - 1 - i);
                int pix = buf[i];
 
                unsigned char *dest = (unsigned char *)lr->data +
                   lr->pitch*true_y + true_x*4;
-               dest[0] = image_palette[pix][2];
-               dest[1] = image_palette[pix][1];
-               dest[2] = image_palette[pix][0];
+               if (pix < palette_start || pix >= (palette_start + palette_colors)) {
+                  al_free(buf);
+                  al_unlock_bitmap(bmp);
+                  al_destroy_bitmap(bmp);
+                  ALLEGRO_ERROR("Invalid image data.\n");
+                  return NULL;
+               }
+               palette_entry* entry = image_palette + pix;
+               dest[0] = (*entry)[2];
+               dest[1] = (*entry)[1];
+               dest[2] = (*entry)[0];
                dest[3] = 255;
             }
 
@@ -410,11 +478,18 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
 
          case 2:
             if (bpp == 32) {
+               unsigned int *ptr;
                if (compressed)
-                  rle_tga_read32((unsigned int *)buf, image_width, f);
+                  ptr = rle_tga_read32((unsigned int *)buf, image_width, f);
                else
-                  raw_tga_read32((unsigned int *)buf, image_width, f);
-
+                  ptr = raw_tga_read32((unsigned int *)buf, image_width, f);
+               if (!ptr) {
+                  al_free(buf);
+                  al_unlock_bitmap(bmp);
+                  al_destroy_bitmap(bmp);
+                  ALLEGRO_ERROR("Invalid image data.\n");
+                  return NULL;
+               }
                for (i = 0; i < image_width; i++) {
                   int true_x = (left_to_right) ? i : (image_width - 1 - i);
                   unsigned char *dest = (unsigned char *)lr->data +
@@ -425,12 +500,12 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
                   int r = buf[i * 4 + 1];
                   int g = buf[i * 4 + 2];
                   int b = buf[i * 4 + 3];
-#else					 
+#else
                   int b = buf[i * 4 + 0];
                   int g = buf[i * 4 + 1];
                   int r = buf[i * 4 + 2];
                   int a = buf[i * 4 + 3];
-#endif                  
+#endif
                   if (premul) {
                      r = r * a / 255;
                      g = g * a / 255;
@@ -444,10 +519,18 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
                }
             }
             else if (bpp == 24) {
+               unsigned char *ptr;
                if (compressed)
-                  rle_tga_read24(buf, image_width, f);
+                  ptr = rle_tga_read24(buf, image_width, f);
                else
-                  raw_tga_read24(buf, image_width, f);
+                  ptr = raw_tga_read24(buf, image_width, f);
+               if (!ptr) {
+                  al_free(buf);
+                  al_unlock_bitmap(bmp);
+                  al_destroy_bitmap(bmp);
+                  ALLEGRO_ERROR("Invalid image data.\n");
+                  return NULL;
+               }
                for (i = 0; i < image_width; i++) {
                   int true_x = (left_to_right) ? i : (image_width - 1 - i);
                   int b = buf[i * 3 + 0];
@@ -463,14 +546,23 @@ ALLEGRO_BITMAP *_al_load_tga_f(ALLEGRO_FILE *f, int flags)
                }
             }
             else {
+               unsigned short *ptr;
                if (compressed)
-                  rle_tga_read16((unsigned short *)buf, image_width, f);
+                  ptr = rle_tga_read16((unsigned short *)buf, image_width, f);
                else
-                  raw_tga_read16((unsigned short *)buf, image_width, f);
+                  ptr = raw_tga_read16((unsigned short *)buf, image_width, f);
+               if (!ptr) {
+                  al_free(buf);
+                  al_unlock_bitmap(bmp);
+                  al_destroy_bitmap(bmp);
+                  ALLEGRO_ERROR("Invalid image data.\n");
+                  return NULL;
+               }
                for (i = 0; i < image_width; i++) {
                   int true_x = (left_to_right) ? i : (image_width - 1 - i);
                   int pix = *((unsigned short *)(buf + i * 2));
-                  int r = _al_rgb_scale_5[(pix >> 10)];
+                  /* TODO - do something with the 1-bit A value (alpha?) */
+                  int r = _al_rgb_scale_5[(pix >> 10) & 0x1F];
                   int g = _al_rgb_scale_5[(pix >> 5) & 0x1F];
                   int b = _al_rgb_scale_5[(pix & 0x1F)];
 
@@ -597,7 +689,7 @@ bool _al_identify_tga(ALLEGRO_FILE *f)
    uint8_t x[4];
    al_fgetc(f); // skip id length
    al_fread(f, x, 4);
-   
+
    if (x[0] > 1) // TGA colormap must be 0 or 1
       return false;
    if ((x[1] & 0xf7) == 0) // type must be 1, 2, 3, 9, 10 or 11
