@@ -8,6 +8,9 @@ static void postprocess_latex(void);
 static void cat(void);
 static void print_sans_backslashes(const char *p);
 
+static void insert_examples(void);
+static char current_api[64];
+
 void make_single_doc(int argc, char *argv[])
 {
    d_init(argc, argv);
@@ -37,6 +40,12 @@ static void preprocess(void)
    dstr line;
 
    while (d_getline(line)) {
+      /* If this is a heading, a new section is about to start.
+         Insert the pending code examples (if any) beforehand. */
+      if (line[0] == '#') {
+         insert_examples();
+      }
+
       /* Raise sections by one level. Top-most becomes the document title. */
       if (line[0] == '#' && raise_sections) {
          if (line[1] == ' ') {
@@ -71,6 +80,7 @@ static void preprocess(void)
             d_printf("~~~~");
          }
          d_printf("\n[Source Code](%s)\n", source);
+         strcpy(current_api, name);
       }
       else if (d_match(line, "^__ALLEGRO_5_CFG")) {
          char *allegro5_cfg = load_allegro5_cfg();
@@ -81,6 +91,8 @@ static void preprocess(void)
          d_print(line);
       }
    }
+   /* Finally insert any examples for the last section */
+   insert_examples();
 }
 
 static void postprocess_latex(void)
@@ -92,7 +104,7 @@ static void postprocess_latex(void)
        * T-Rex doesn't seem to backtrack properly if we write "(sub)*".
        */
       if (d_match(line,
-	    "\\\\(sub)?(sub)?(sub)?(sub)?section\\{((al|ALLEGRO)[^}]+)")) {
+            "\\\\(sub)?(sub)?(sub)?(sub)?section\\{((al|ALLEGRO)[^}]+)")) {
          d_print(line);
          d_printf("\\label{");
          print_sans_backslashes(d_submatch(5));
@@ -133,4 +145,38 @@ static void print_sans_backslashes(const char *p)
    }
 }
 
+static void insert_examples(void) {
+   if (*current_api) {
+      const char* exs = lookup_example(current_api);
+      if (exs) {
+         /* This will be of the form "FILE:LINE FILE:LINE FILE:LINE " */
+         dstr items;
+         char* pitem = strcpy(items, exs);
+         d_print("Examples:\n");
+         char* item;
+         for (item = strtok(pitem, " "); item; item = strtok(NULL, " ")) {
+            dstr buffer;
+            char* colon = strchr(item, ':');
+            if (colon) {
+               char* filename = item;
+               char* line = colon + 1;
+               *colon = '\0';
+               dstr base;
+               d_basename(filename, NULL, base);
+               d_printf("* [%s](%s)\n",
+                        base,
+                        example_source(buffer, filename, line));
+            }
+         }
+         d_print("");
+      }
+      strcpy(current_api, "");
+   }
+}
+
+/* Local Variables: */
+/* c-basic-offset: 3 */
+/* indent-tabs-mode: nil */
+/* End: */
+
 /* vim: set sts=3 sw=3 et: */
