@@ -63,8 +63,11 @@ static void go(void)
    al_destroy_display(d2);
 }
 
-int main(void)
+int main(int argc, char **argv)
 {
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
