@@ -51,7 +51,7 @@ void _al_set_bitmap_shader_field(ALLEGRO_BITMAP *bmp, ALLEGRO_SHADER *shader);
 void _al_register_shader_bitmap(ALLEGRO_SHADER *shader, ALLEGRO_BITMAP *bmp);
 void _al_unregister_shader_bitmap(ALLEGRO_SHADER *shader, ALLEGRO_BITMAP *bmp);
 
-ALLEGRO_SHADER *_al_create_default_shader(int display_flags);
+ALLEGRO_SHADER *_al_create_default_shader(ALLEGRO_DISPLAY *display);
 
 #ifdef ALLEGRO_CFG_SHADER_GLSL
 ALLEGRO_SHADER *_al_create_shader_glsl(ALLEGRO_SHADER_PLATFORM platform);
@@ -59,7 +59,7 @@ void _al_set_shader_glsl(ALLEGRO_DISPLAY *display, ALLEGRO_SHADER *shader);
 #endif
 
 #ifdef ALLEGRO_CFG_SHADER_HLSL
-ALLEGRO_SHADER *_al_create_shader_hlsl(ALLEGRO_SHADER_PLATFORM platform);
+ALLEGRO_SHADER *_al_create_shader_hlsl(ALLEGRO_SHADER_PLATFORM platform, int shader_model);
 void _al_set_shader_hlsl(ALLEGRO_DISPLAY *display, ALLEGRO_SHADER *shader);
 #endif
 
