@@ -50,6 +50,7 @@ struct system_data_t {
    ALLEGRO_USTR *user_lib_name;
    ALLEGRO_USTR *resources_dir;
    ALLEGRO_USTR *data_dir;
+   ALLEGRO_USTR *temp_dir;
    ALLEGRO_USTR *apk_path;
    ALLEGRO_USTR *model;
    ALLEGRO_USTR *manufacturer;
@@ -220,11 +221,13 @@ JNI_FUNC(bool, AllegroActivity, nativeOnCreate, (JNIEnv *env, jobject obj))
    system_data.user_lib_name = _jni_callStringMethod(env, system_data.activity_object, "getUserLibName", "()Ljava/lang/String;");
    system_data.resources_dir = _jni_callStringMethod(env, system_data.activity_object, "getResourcesDir", "()Ljava/lang/String;");
    system_data.data_dir = _jni_callStringMethod(env, system_data.activity_object, "getPubDataDir", "()Ljava/lang/String;");
+   system_data.temp_dir = _jni_callStringMethod(env, system_data.activity_object, "getTempDir", "()Ljava/lang/String;");
    system_data.apk_path = _jni_callStringMethod(env, system_data.activity_object, "getApkPath", "()Ljava/lang/String;");
    system_data.model = _jni_callStringMethod(env, system_data.activity_object, "getModel", "()Ljava/lang/String;");
    system_data.manufacturer = _jni_callStringMethod(env, system_data.activity_object, "getManufacturer", "()Ljava/lang/String;");
    ALLEGRO_DEBUG("resources_dir: %s", al_cstr(system_data.resources_dir));
    ALLEGRO_DEBUG("data_dir: %s", al_cstr(system_data.data_dir));
+   ALLEGRO_DEBUG("temp_dir: %s", al_cstr(system_data.temp_dir));
    ALLEGRO_DEBUG("apk_path: %s", al_cstr(system_data.apk_path));
    ALLEGRO_DEBUG("model: %s", al_cstr(system_data.model));
    ALLEGRO_DEBUG("manufacturer: %s", al_cstr(system_data.manufacturer));
@@ -503,6 +506,8 @@ static bool android_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO *info)
 
    ALLEGRO_DEBUG("Monitor Info: %d:%d", info->x2, info->y2);
 
+   _jni_callv(env, DeleteLocalRef, rect);
+
    return true;
 }
 
@@ -533,6 +538,7 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_android_interface()
    android_vt = al_malloc(sizeof *android_vt);
    memset(android_vt, 0, sizeof *android_vt);
 
+   android_vt->id = ALLEGRO_SYSTEM_ID_ANDROID;
    android_vt->initialize = android_initialize;
    android_vt->get_display_driver = _al_get_android_display_driver;
    android_vt->get_keyboard_driver = _al_get_android_keyboard_driver;
@@ -544,6 +550,9 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_android_interface()
    android_vt->get_path = _al_android_get_path;
    android_vt->shutdown_system = android_shutdown_system;
    android_vt->inhibit_screensaver = android_inhibit_screensaver;
+   android_vt->get_time = _al_unix_get_time;
+   android_vt->rest = _al_unix_rest;
+   android_vt->init_timeout = _al_unix_init_timeout;
 
    return android_vt;
 }
@@ -559,6 +568,10 @@ ALLEGRO_PATH *_al_android_get_path(int id)
          break;
 
       case ALLEGRO_TEMP_PATH:
+         /* path to the application cache */
+         path = al_create_path_for_directory(al_cstr(system_data.temp_dir));
+         break;
+
       case ALLEGRO_USER_DATA_PATH:
       case ALLEGRO_USER_HOME_PATH:
       case ALLEGRO_USER_SETTINGS_PATH:
@@ -652,4 +665,27 @@ jobject al_android_get_activity(void)
    return _al_android_activity_object();
 }
 
+/* Function: al_android_open_fd
+ */
+int al_android_open_fd(const char *uri, const char *mode)
+{
+   JNIEnv *env = _al_android_get_jnienv();
+   jobject activity = _al_android_activity_object();
+
+   jstring juri = _jni_call(env, jstring, NewStringUTF, uri != NULL ? uri : ""); /* content:// or file:// */
+   jstring jmode = _jni_call(env, jstring, NewStringUTF, mode != NULL ? mode : ""); /* "r", "w", "rw", "wa", "wt", "rwt" */
+
+   int fd = _jni_callIntMethodV(env, activity, "openFileDescriptor", "(Ljava/lang/String;Ljava/lang/String;)I", juri, jmode);
+
+   _jni_callv(env, DeleteLocalRef, jmode);
+   _jni_callv(env, DeleteLocalRef, juri);
+
+   if (fd == -1)
+      al_set_errno(ENOENT);
+   else if (fd < 0)
+      al_set_errno(ENOTSUP);
+
+   return fd;
+}
+
 /* vim: set sts=3 sw=3 et: */
