@@ -1,6 +1,6 @@
-/*         ______   ___    ___ 
+/*         ______   ___    ___
  *        /\  _  \ /\_ \  /\_ \
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -602,6 +602,41 @@ void al_acknowledge_drawing_resume(ALLEGRO_DISPLAY *display)
    }
 }
 
+/* Function: al_get_render_state
+ */
+int al_get_render_state(ALLEGRO_RENDER_STATE state)
+{
+   ALLEGRO_DISPLAY *display = al_get_current_display();
+
+   if (!display)
+      return -1;
+
+   switch (state) {
+      case ALLEGRO_ALPHA_TEST:
+         return display->render_state.alpha_test;
+         break;
+      case ALLEGRO_WRITE_MASK:
+         return display->render_state.write_mask;
+         break;
+      case ALLEGRO_DEPTH_TEST:
+         return display->render_state.depth_test;
+         break;
+      case ALLEGRO_DEPTH_FUNCTION:
+         return display->render_state.depth_function;
+         break;
+      case ALLEGRO_ALPHA_FUNCTION:
+         return display->render_state.alpha_function;
+         break;
+      case ALLEGRO_ALPHA_TEST_VALUE:
+         return display->render_state.alpha_test_value;
+         break;
+      default:
+         ALLEGRO_ERROR("Unknown state to retrieve: %d\n", state);
+         return -1;
+         break;
+   }
+}
+
 /* Function: al_set_render_state
  */
 void al_set_render_state(ALLEGRO_RENDER_STATE state, int value)
@@ -631,7 +666,7 @@ void al_set_render_state(ALLEGRO_RENDER_STATE state, int value)
          display->render_state.alpha_test_value = value;
          break;
       default:
-         ALLEGRO_WARN("unknown state to change: %d\n", state);
+         ALLEGRO_WARN("Unknown state to change: %d\n", state);
          break;
    }
 
@@ -667,4 +702,22 @@ void al_apply_window_constraints(ALLEGRO_DISPLAY *display, bool onoff)
       display->vt->apply_window_constraints(display, onoff);
 }
 
+/* Function: al_get_display_adapter
+ */
+int al_get_display_adapter(ALLEGRO_DISPLAY *display)
+{
+   int x, y, adapter, num_adapters;
+   al_get_window_position(display, &x, &y);
+   num_adapters = al_get_num_video_adapters();
+   for (adapter = 0; adapter < num_adapters; adapter++) {
+      ALLEGRO_MONITOR_INFO mi;
+      al_get_monitor_info(adapter, &mi);
+      if (x >= mi.x1 && x < mi.x2 && y >= mi.y1 && y < mi.y2) {
+         return adapter;
+      }
+   }
+   return -1;
+}
+
+
 /* vim: set sts=3 sw=3 et: */
