@@ -20,9 +20,6 @@
 #ifndef _LARGEFILE_SOURCE
    #define _LARGEFILE_SOURCE
 #endif
-#ifndef _LARGEFILE_SOURCE64
-   #define _LARGEFILE_SOURCE64
-#endif
 #ifndef _FILE_OFFSET_BITS
    #define _FILE_OFFSET_BITS 64
 #endif
