@@ -30,7 +30,10 @@ Note that setting the view non-opaque will only work if the EAGL surface has an
     */
    _AL_LIST*            touch_list;
    NSMutableIndexSet*   touch_id_set;
+   UITouch*             primary_touch;
    int                  next_free_touch_id;
+
+   float                scale;
 }
 
 @property (nonatomic, retain) EAGLContext *context;
@@ -44,4 +47,5 @@ Note that setting the view non-opaque will only work if the EAGL surface has an
 - (void)set_allegro_display:(ALLEGRO_DISPLAY *)display;
 - (BOOL) createFramebuffer;
 - (void) destroyFramebuffer;
+- (BOOL)orientation_supported:(UIInterfaceOrientation)o;
 @end
