@@ -6,6 +6,12 @@
 #include "allegro5/internal/aintern_xdisplay.h"
 #include "allegro5/internal/aintern_xsystem.h"
 
+#ifdef ALLEGRO_RASPBERRYPI
+#include "allegro5/internal/aintern_raspberrypi.h"
+#define ALLEGRO_SYSTEM_XGLX ALLEGRO_SYSTEM_RASPBERRYPI
+#define ALLEGRO_DISPLAY_XGLX ALLEGRO_DISPLAY_RASPBERRYPI
+#endif
+
 #include <X11/cursorfont.h>
 
 #ifdef ALLEGRO_XWINDOWS_WITH_XCURSOR
